@@ -645,12 +645,26 @@ class _TestObject(object):
         self.assertRaises(ValueError, fail)
 
     def test_object_dict_syntax(self):
-        obj = MyObj(foo=123, bar='bar')
+        obj = MyObj(foo=123, bar=u'text')
         self.assertEqual(obj['foo'], 123)
-        self.assertEqual(sorted(obj.items(), key=lambda x: x[0]),
-                         [('bar', 'bar'), ('foo', 123)])
-        self.assertEqual(sorted(list(obj.iteritems()), key=lambda x: x[0]),
-                         [('bar', 'bar'), ('foo', 123)])
+        self.assertIn('bar', obj)
+        self.assertNotIn('missing', obj)
+        self.assertEqual(sorted(iter(obj)),
+                         ['bar', 'foo'])
+        self.assertEqual(sorted(obj.keys()),
+                         ['bar', 'foo'])
+        self.assertEqual(sorted(obj.iterkeys()),
+                         ['bar', 'foo'])
+        self.assertEqual(sorted(obj.values(), key=str),
+                         [123, u'text'])
+        self.assertEqual(sorted(obj.itervalues(), key=str),
+                         [123, u'text'])
+        self.assertEqual(sorted(obj.items()),
+                         [('bar', u'text'), ('foo', 123)])
+        self.assertEqual(sorted(list(obj.iteritems())),
+                         [('bar', u'text'), ('foo', 123)])
+        self.assertEqual(dict(obj),
+                         {'foo': 123, 'bar': u'text'})
 
     def test_load(self):
         obj = MyObj()
@@ -1000,6 +1014,20 @@ class _TestObject(object):
             obj.obj_to_primitive('1.0')
             self.assertTrue(mock_mc.called)
 
+    def test_delattr(self):
+        obj = MyObj(bar='foo')
+        del obj.bar
+
+        # Should appear unset now
+        self.assertFalse(obj.obj_attr_is_set('bar'))
+
+        # Make sure post-delete, references trigger lazy loads
+        self.assertEqual('loaded!', getattr(obj, 'bar'))
+
+    def test_delattr_unset(self):
+        obj = MyObj()
+        self.assertRaises(AttributeError, delattr, obj, 'bar')
+
     def test_obj_make_compatible_on_list_base(self):
         @base.VersionedObjectRegistry.register_if(False)
         class MyList(base.ObjectListBase, base.VersionedObject):
